import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import { useContext, useCallback, useMemo } from 'react';
import { ResolveContext } from './context';
import { useClient } from './use-client';

var useViewModel = function useViewModel(modelName, aggregateIds, stateChangeCallback, queryOptions) {
  var log = console;
  var context = useContext(ResolveContext);
  var client = useClient();
  log.debug("searching for view model metadata");
  var viewModels = context.viewModels;
  var viewModel = viewModels.find(function (_ref) {
    var name = _ref.name;
    return name === modelName;
  });

  if (!viewModel) {
    var _error = Error("View model " + modelName + " not exist within context");

    log.error(_error.message);
    throw _error;
  }

  log.debug("view model metadata found");
  var closure = useMemo(function () {
    return {
      state: viewModel.projection.Init ? viewModel.projection.Init() : null
    };
  }, []);
  var setState = useCallback(function (state) {
    closure.state = state;
    stateChangeCallback(closure.state);
  }, []);
  var queryState = useCallback( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
    var result;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            log.debug("querying view model state");
            _context.next = 3;
            return client.query({
              name: modelName,
              aggregateIds: aggregateIds,
              args: {}
            }, queryOptions);

          case 3:
            result = _context.sent;
            log.debug("view model state arrived");

            if (result) {
              log.debug(result.data);
              setState(result.data);
            } else {
              log.debug("query have not result");
            }

          case 6:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  })), []);
  var applyEvent = useCallback(function (event) {
    log.debug("applying event [" + event.type + "]");
    setState(viewModel.projection[event.type](closure.state, event));
  }, []);
  var connect = useCallback(function (done) {
    var asyncConnect = /*#__PURE__*/function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var subscribe, subscription;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                log.debug("connecting view model");
                _context2.next = 3;
                return queryState();

              case 3:
                log.debug("subscribing to incoming events");
                subscribe = client.subscribe(modelName, aggregateIds, function (event) {
                  return applyEvent(event);
                }, undefined, function () {
                  return queryState();
                });
                _context2.next = 7;
                return subscribe;

              case 7:
                subscription = _context2.sent;
                log.debug("subscribed successfully");

                if (subscription) {
                  closure.subscription = subscription;
                }

                return _context2.abrupt("return", subscription);

              case 11:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      return function asyncConnect() {
        return _ref3.apply(this, arguments);
      };
    }();

    if (typeof done !== 'function') {
      return asyncConnect();
    }

    asyncConnect().then(function (result) {
      return done(null, result);
    })["catch"](function (error) {
      return done(error, null);
    });
    return undefined;
  }, []);
  var dispose = useCallback(function (done) {
    var asyncDispose = /*#__PURE__*/function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!closure.subscription) {
                  _context3.next = 3;
                  break;
                }

                _context3.next = 3;
                return client.unsubscribe(closure.subscription);

              case 3:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      return function asyncDispose() {
        return _ref4.apply(this, arguments);
      };
    }();

    if (typeof done !== 'function') {
      return asyncDispose();
    }

    asyncDispose().then(function () {
      return done();
    })["catch"](function (error) {
      return done(error);
    });
    return undefined;
  }, []);
  return useMemo(function () {
    return {
      connect: connect,
      dispose: dispose
    };
  }, []);
};

export { useViewModel };
//# sourceMappingURL=use-view-model.js.map